﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Text;
using SoftwareIDM.Uplift;

namespace SoftwareIDM.Uplift.MailQueue
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Processing Mail Queue");
            Config.ExeConfigPath = @".\MailQueue.exe.config";
            List<long> ids = new List<long>();
            var table = Config.App.AppSettings["table"];
            foreach (var row in SqlHelper.Query("FIMSupport", String.Format("SELECT * FROM {0} WHERE sent IS NULL ORDER BY queued ASC", SqlQuote.quote(table))))
            {
                if (!send(row["recipients"].ToString(), row["subject"].ToString(), row["message"].ToString()))
                {
                    break;
                }

                ids.Add((long)row["id"]);
                if (ids.Count % 10 == 0)
                {
                    Console.Write(".");
                }
            }

            foreach (long id in ids)
            {
                SqlHelper.NonQuery("FIMSupport", String.Format("UPDATE {0} SET sent=GETDATE() WHERE id=@id", SqlQuote.quote(table)),
                    new SqlParameter("@id", id));
            }

            Console.WriteLine("Sent {0} email messages.", ids.Count);
        }

        static SmtpClient client;

        static bool send(string recipients, string subject, string message)
        {
            try
            {
                MailMessage msg = new MailMessage(Config.App.AppSettings["from"], recipients, subject, message);
                if (Config.App.AppSettings.ContainsKey("bcc"))
                {
                    msg.Bcc.Add(Config.App.AppSettings["bcc"]);
                }

                if (client == null)
                {
                    client = new SmtpClient(Config.App.AppSettings["host"]);
                    if (Config.App.AppSettings.ContainsKey("port"))
                    {
                        var port = Config.App.AppSettings["port"];
                        if (!String.IsNullOrEmpty(port))
                        {
                            client.Port = int.Parse(port);
                        }

                        if (client.Port != 25)
                        {
                            client.EnableSsl = true;
                        }
                    }

                    if (Config.App.AppSettings.ContainsKey("user"))
                    {
                        var user = Config.App.AppSettings["user"];
                        if (!String.IsNullOrEmpty(user))
                        {
                            client.Credentials = new NetworkCredential(user, Config.App.AppSettings["password"]);
                        }
                    }
                }

                client.Send(msg);

                LogWriter.writeEntry("MailQueue", "Sent Message", "MailQueue", LogType.Info, "Sent message {0} to {1}", subject, recipients);
            }
            catch (Exception e)
            {
                Console.WriteLine("Error sending message: {0}", e.Message);
                LogWriter.writeEntry("MailQueue", "Error Sending Message", "MailQueue", LogType.Error, "Error sending message: {0}\r\n{1}", e.Message, e.StackTrace);
                return false;
            }

            return true;
        }
    }
}
