﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Data.SqlClient;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml.Linq;

namespace SoftwareIDM.Uplift
{
    public static class Mail
    {
        /// <summary>
        /// Used to create table if it's not already present.
        /// </summary>
        private static bool verified;
        private static void VerifyTable(SqlConnection conn)
        {
            if (!verified)
            {
                string create_table = @"
                IF NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'{0}') AND type in (N'U'))
                BEGIN
                CREATE TABLE {0}(
	                [id] [bigint] IDENTITY(1,1) NOT NULL,
	                [recipients] [nvarchar](max) NOT NULL,
	                [subject] [nvarchar](150) NOT NULL,
	                [message] [nvarchar](max) NOT NULL,
	                [sent] [datetime] NULL,
	                [queued] [datetime] NOT NULL,
                CONSTRAINT {1} PRIMARY KEY CLUSTERED 
                (
	                [id] ASC
                )WITH (PAD_INDEX = OFF, IGNORE_DUP_KEY = OFF) ON [PRIMARY]
                ) ON [PRIMARY]
                END
                ";

                create_table = String.Format(create_table, SqlQuote.quote(Config.App.Mail.table), SqlQuote.quote("PK_" + Config.App.Mail.table));

                SqlCommand cmd = new SqlCommand(create_table, conn);
                cmd.ExecuteNonQuery();
                verified = true;
            }
        }

        /// <summary>
        /// Get template from EmailTemplates.xml for constructing mail message.
        /// </summary>
        public static MailTemplate GetTemplate(string name)
        {
            return Config.App.Mail.templates[name];
        }

        /// <summary>
        /// Write an entry to the mail queue.
        /// </summary>
        /// <param name="recipients">';' separated list of recipients.</param>
        /// <param name="subject">Message subject</param>
        /// <param name="message">Message body</param>
        /// <param name="args">args to format into message with String.Format or Dictionary lookup</param>
        public static void Queue(string recipients, string subject, string message, params object[] args)
        {
            using (var conn = new SqlConnection(Config.App.Mail.connectionString))
            {
                conn.Open();
                VerifyTable(conn);

                var exists = new SqlCommand(
                    String.Format("SELECT COUNT(0) FROM {0} WHERE recipients=@r AND subject=@s AND message=@m AND sent IS NULL",
                    SqlQuote.quote(Config.App.Mail.table)), conn);
                exists.Parameters.AddWithValue("@r", recipients);
                exists.Parameters.AddWithValue("@s", Template(subject, args));
                exists.Parameters.AddWithValue("@m", Template(message, args));

                if ((int)exists.ExecuteScalar() == 0)
                {
                    var cmd = new SqlCommand(
                        String.Format("INSERT INTO {0} (recipients, subject, message, queued) VALUES (@r, @s, @m, @q)",
                        SqlQuote.quote(Config.App.Mail.table)), conn);
                    cmd.Parameters.AddWithValue("@r", recipients);
                    cmd.Parameters.AddWithValue("@s", Template(subject, args));
                    cmd.Parameters.AddWithValue("@m", Template(message, args));
                    cmd.Parameters.AddWithValue("@q", DateTime.Now);
                    cmd.ExecuteNonQuery();
                }
            }
        }

        private static string Template(string value, params object[] args)
        {
            var regex = new Regex(@"(?<!\\)\[(.+?)]");
            var paramList = new List<object>();
            var holos = new List<IDictionary<string, AttributeValue>>();
            var changes = new List<IDictionary<string, Change>>();
            foreach (var obj in args)
            {
                if (obj is IDictionary<string, AttributeValue>)
                {
                    holos.Add((IDictionary<string, AttributeValue>)obj);
                }
                else if (obj is IDictionary<string, Change>)
                {
                    changes.Add((IDictionary<string, Change>)obj);
                }
                else
                {
                    paramList.Add(obj);
                }
            }

            foreach (Match match in regex.Matches(value))
            {
                var key = match.Groups[1].Value;
                if (key.EndsWith("OldValue"))
                {
                    var field = key.Replace(".OldValue", "");
                    foreach (var set in changes)
                    {
                        if (set.ContainsKey(field))
                        {
                            value = value.Replace(match.Groups[0].Value, changes[0][field].OldValue.ToString());
                            break;
                        }
                    }
                }
                else if (key.EndsWith("NewValue"))
                {
                    var field = key.Replace(".NewValue", "");
                    foreach (var set in changes)
                    {
                        if (set.ContainsKey(field))
                        {
                            value = value.Replace(match.Groups[0].Value, changes[0][field].NewValue.ToString());
                            break;
                        }
                    }
                }
                else
                {
                    foreach (var set in holos)
                    {
                        if (set.ContainsKey(match.Groups[1].Value))
                        {
                            value = value.Replace(match.Groups[0].Value, set[key].ToString());
                            break;
                        }
                    }
                }

                value = value.Replace(match.Groups[0].Value, "");
            }

            return String.Format(value, paramList.ToArray());
        }
    }
}
