﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;

namespace SoftwareIDM.Uplift
{
    public static class SqlHelper
    {
        public static object Scalar(SqlConnection connection, SqlTransaction tran, string query, params SqlParameter[] parameters)
        {
            SqlCommand cmd = new SqlCommand(query, connection);
            cmd.Parameters.AddRange(parameters);
            if (tran != null)
            {
                cmd.Transaction = tran;
            }

            return cmd.ExecuteScalar();
        }

        public static object Scalar(string connectionStringKey, string query, params SqlParameter[] parameters)
        {
            return Scalar(connectionStringKey, query, IsolationLevel.ReadCommitted, parameters);
        }

        public static object Scalar(string connectionStringKey, string query, IsolationLevel isolationLevel, params SqlParameter[] parameters)
        {
            var connString = Config.App.ConnectionStrings[connectionStringKey].ConnectionString;
            using (var conn = new SqlConnection(connString))
            {
                conn.Open();
                var tran = conn.BeginTransaction(isolationLevel);
                SqlCommand cmd = new SqlCommand(query, conn);
                cmd.Transaction = tran;
                cmd.Parameters.AddRange(parameters);
                var ret = cmd.ExecuteScalar();
                tran.Commit();
                return ret;
            }
        }

        public static IEnumerable<SqlDataReader> Query(SqlConnection connection, SqlTransaction tran, string query, params SqlParameter[] parameters)
        {
            SqlCommand cmd = new SqlCommand(query, connection);
            cmd.Parameters.AddRange(parameters);
            if (tran != null)
            {
                cmd.Transaction = tran;
            }

            using (var reader = cmd.ExecuteReader())
            {
                while (reader.Read())
                {
                    yield return reader;
                }
            }
        }

        public static IEnumerable<SqlDataReader> Query(string connectionStringKey, string query, params SqlParameter[] parameters)
        {
            return Query(connectionStringKey, query, IsolationLevel.ReadCommitted, parameters);
        }

        public static IEnumerable<SqlDataReader> Query(string connectionStringKey, string query, IsolationLevel isolationLevel, params SqlParameter[] parameters)
        {
            var connString = Config.App.ConnectionStrings[connectionStringKey].ConnectionString;
            using (var conn = new SqlConnection(connString))
            {
                conn.Open();
                var tran = conn.BeginTransaction(isolationLevel);
                SqlCommand cmd = new SqlCommand(query, conn, tran);
                cmd.Transaction = tran;
                cmd.Parameters.AddRange(parameters);
                using (var reader = cmd.ExecuteReader())
                {
                    while (reader.Read())
                    {
                        yield return reader;
                    }
                }

                tran.Commit();
            }
        }

        public static void NonQuery(SqlConnection connection, SqlTransaction tran, string query, params SqlParameter[] parameters)
        {
            SqlCommand cmd = new SqlCommand(query, connection);
            cmd.Parameters.AddRange(parameters);
            if (tran != null)
            {
                cmd.Transaction = tran;
            }

            cmd.ExecuteNonQuery();
        }

        public static void NonQuery(string connectionStringKey, string query, params SqlParameter[] parameters)
        {
            NonQuery(connectionStringKey, query, IsolationLevel.ReadCommitted, parameters);
        }

        public static void NonQuery(string connectionStringKey, string query, IsolationLevel isolationLevel, params SqlParameter[] parameters)
        {
            var connString = Config.App.ConnectionStrings[connectionStringKey].ConnectionString;
            using (var conn = new SqlConnection(connString))
            {
                conn.Open();
                var tran = conn.BeginTransaction(isolationLevel);
                SqlCommand cmd = new SqlCommand(query, conn);
                cmd.Transaction = tran;
                cmd.Parameters.AddRange(parameters);
                cmd.ExecuteNonQuery();
                tran.Commit();
            }
        }
    }

    /// <summary>
    /// Class to escape SQL table and column names. Based on documentation for TSQL QUOTENAME function
    /// </summary>
    public static class SqlQuote
    {
        /// <summary>
        /// Quote val with square braces as column name
        /// </summary>
        public static string quote(string val)
        {
            val = val.Replace("]", "]]");
            if (val.StartsWith("[") && val.EndsWith("]]"))
                return val.Substring(0, val.Length - 1);
            else
                return "[" + val + "]";
        }

        /// <summary>
        /// Quote val with square braces. 
        /// If splitdots is true, each part of the '.' separated string will be treated as a separate 
        /// part of the identifier path.
        /// </summary>
        /// <param name="val"></param>
        /// <param name="splitdots"></param>
        /// <returns></returns>
        public static string quote(string val, bool splitdots)
        {
            if (splitdots)
            {
                string[] vals = val.Split('.');
                StringBuilder ret = new StringBuilder();
                for (int i = 0; i < vals.Length; i++)
                {
                    if (vals[i].Length > 0)
                        ret.Append(quote(vals[i]) + ".");
                    else
                        ret.Append(".");
                }
                return ret.ToString().TrimEnd('.');
            }
            else
            {
                return quote(val);
            }
        }
    }
}
